/* Classes and functions pertaining to WLC generation
 * and analysis. */

#ifndef H_WLC2D
#define H_WLC2D

#include <iostream>       // Input/output
#include <Eigen/Core>     // Linear algebra
#include <Eigen/Geometry> // Rotation 2D
#include <cmath>          // General math operations
#include <cstdlib>        // Standard library
#include <string>         // Strings
#include <vector>         // Vectors for storing data
#include <random>         // Generating random numbers
#include <stdexcept>      // Throwing exceptions
#include <fstream>        // Saving/Loading data
#include <limits>         // Limits of data types
#include <iomanip>        // std::setprecision

#include "Path2D.h"
#include "RgDict.h"
#include "Misc.h"

class WLC2D: public Path2D
/* A 2D wormlike chain.
 * 
 * Member variables:
 * -----------------
 * pathLength : int
 *     The number of segments in the chain.
 * persisLength: double
 *     The persistence length in units of chain segments.
 * initPoint : pointer to Eigen::Vector2d
 *     The coordinates of the first point of the chain.
 * path : std::vector<Eigen::Vector2d>
 *     Inherited from Path2D. The set of all segments in the chain.
 */

{
public:

    WLC2D(int in_pathLength,
           double in_persisLength, 
           Eigen::Vector2d * in_initPoint);
    /* Constructor */

    void makePath(int in_pathLength);
    /*  Create the wormlike chain.
     *  
     *  The wormlike chain is created by first choosing the sizes of
     *  the small, random displacements along a line tangent to a point
     *  on the surface of the unit circle defined by the vector
     *  currPoint. The distribution for the sizes is given by the
     *  Boltzmann statistics for a semiflexible rod bending by an
     *  angle due to interaction with its thermal environment.
     *  
     *  Each successive segment of the chain is generated by rotating the
     *  current segment about the +z axis by a random angle. This angle is
     *  drawn from a Gaussian probability distribution function whose
     *  standard deviation is related to the persistence length of the chain.
     *
     *  Th iInitial point to start the polymer is determined by initPoint,
     *  which is set when initializing the class WLC. */
};

#endif